var irc = require('irc');
var express = require('express');
var bodyParser = require('body-parser');

var config = require('./config');

var client = new irc.Client(config.server, config.nick, {
    //debug: true,
    channels: config.channels,
    userName: config.userName,
    realName: config.realName,
    retryDelay: 120000
});

var say = client.say.bind(client);

var app = express();

app.use(bodyParser.json()); // for parsing application/json
app.use(bodyParser.urlencoded({ extended: true })); // for parsing application/x-www-form-urlencoded

function formatCommit(commit) {
    return commit.message + ' ' + commit.url;
}

var handlers = {

    push: function(body) {
        var user = body.user_name;
        var repoName = body.repository.name;
        var commits = body.commits;

        var msg = ['push on ' + repoName + ': ' + user + ' pushed ' + commits.length + ' commits.'];

        if (commits.length < 4) {
            msg = msg.concat(commits.map(formatCommit));
        } else {
            msg.push(formatCommit(commits[0]));
            msg.push('...');
            msg.push(formatCommit(commits[commits.length - 1]));
        }

        return msg;
    }

};

app.post('/', function(req, res) {
    var body = req.body || {};

    var msgs = null;
    if (body.object_kind && handlers[body.object_kind])
        msgs = handlers[body.object_kind](body);

    if (msgs) {
        if (msgs instanceof Array) {
            for (var i = 0; i < msgs.length; i++)
                say(config.channels, msgs[i]);
        } else {
            say(config.channels, msgs);
        }
    }

    res.sendStatus(200);
});

app.listen(config.port, config.hostname, function() {
    console.log('gitlab-to-irc running.');
});
